stacksize(50000000);

//if You want, You can change the parameters
//styles = ["pop","clr","cla","pnk","rhh","hrm","tra"];
//numberOfFiles = 50;

//the line that executes the code
analizeAll(styles, numberOfFiles, 8000, 1, 512, 80);

function analizeAll(styles, numberOfFiles, hz, sec, step, percent)
  fid = mopen("output34flux.csv", "w");
  mfprintf(fid, "size, avCentroid, avCentroid_std, cPeaksOverPRate, flux, flux_std, wavAvDif, wavAvDif_std, wavAvAmpl, wavAvAmpl_std, wavZero, wavZero_std, lowEnergy, lowFreq, lowFreq_std, highFreq, highFreq_std, rateHL, rateHL_std, maxH, maxH_std, maxL, maxL_std, maxHL, maxHL_std, rolloff, rolloff_std, a0, p0, a1, p1, a2, p2, a3, p3, style\n");
  mclose(fid);
  
  len = size(styles)
  for i=1:len(2)
   analizeStyle(styles(i), numberOfFiles, hz, sec, step, percent);
  end  
endfunction

function analizeStyle(fileNamePattern, numberOfFiles, hz, sec, step, percent)
  for i=1:numberOfFiles
    fileName = fileNamePattern + string(i) + ".wav";
    analizeFile(fileName, hz, sec, step, percent);
  end
endfunction

function analizeFile(fileName, hz, sec, step, percent)
  fail=wavread(fileName);
  //'mono' signal
  wave = fail(1,:)
  
  times = floor(sec*hz/step);
  startTimes = generateStartTimes(wave, step, times);
  
  rstep = 17000;
  
  fid = mopen("output34flux.csv", "a");
  for i=1:length(startTimes)
    mfprintf(fid, "%d", length(wave));
    analizeByCentroids(wave, times, step, percent, startTimes(i));
    //analizeByWave(wave, times, step);
    analizeFileByWave(wave, times, step, startTimes(i));
    //analizeByFft(wave, times, step);
    analizeFileByFft(wave, times, step, startTimes(i));
    analizeFileByRythm(wave, i, rstep);
    style = strsplit(fileName,3);
    mfprintf(fid, ", %s", style(1));
    mfprintf(fid, "\n");
  end;
  mclose(fid);
endfunction

function s=generateStartTimes(wave, step, times)
  //30points
  s=[];
  start = round(length(wave)/2-15*step*times);
  for i=1:30
    s(i)=start+step*times*i;
  end
endfunction
//######

function analizeByCentroids(wave, times, step, percent, start)
  centroidsVect = centroidsVector(wave, times, step, start);
  
  mfprintf(fid, ", %f", averageCentroid(centroidsVect, times));
  mfprintf(fid, ", %f", stdev(centroidsVect));
  mfprintf(fid, ", %f", centroidPeaksOverPRate(centroidsVect, percent));
  //mfprintf(fid, ", %f", centroidAverageDiffer(centroidsVect));
  mfprintf(fid, ", %f", mean(flux(centroidsVect)));
  mfprintf(fid, ", %f", stdev(flux(centroidsVect)));
endfunction


//############Centroids########################

function c=centroid(wave)
    s = sum(wave);
    if s == 0 then
       c = 0;
    else
      wave = wave/sum(wave);
      f = 0:(length(wave)-1)
      c = mean(f.*wave)
    end;
endfunction

//takes %sec starting from the middle point of the song, 
//takes centoids for every %step-sample subpart of it
//%step should be %step = 2^n
function c = centroidsVector(wave, times, step, start)
  //start = round(length(wave)/2);
  c = zeros(1, times);
  for i=1:times
    w = wave(1,(start + step*i):(start + step*i+(step-1)));
    fw = abs(fft(w));
    fw1 = fw(1:step/2);
    c(i) = centroid(fw1);
  end;
endfunction

function a=averageCentroid(centroidsVect, times)
    a= sum(centroidsVect)/times;
endfunction 


/////


function p=centroidsPeaksSum(centroidsVect)
    p = 0;
    for i=1:length(centroidsVect)-2
      if centroidsVect(i)<centroidsVect(i+1) then
        if centroidsVect(i+1)>centroidsVect(i+2) then
          p = p+1;
        end;
      end;
    end
endfunction

function p=centroidPeaksOverP(centroidsVect, percent)
    dif = max(centroidsVect)-min(centroidsVect);
    //list of number of potential peaks at needed level (p%)
    f = find(centroidsVect > min(centroidsVect)+((percent*dif)/100))
    p = 0;
    s= size(centroidsVect);
    for i=1:length(f),
      j = f(i)
      if j==1 then
        p=p;
      elseif j == s(2) then 
          p=p;
      elseif centroidsVect(j-1)<centroidsVect(j) then
        if centroidsVect(j)>centroidsVect(j+1) then
          p = p+1;
        end,
      end;
    end
endfunction  

function a=centroidPeaksOverPRate(centroidsVect, percent)
    over = centroidPeaksOverP(centroidsVect, percent);
    all= centroidsPeaksSum(centroidsVect);
    if all == 0 then a = 0;
    else
    a = over/all;
    end;
endfunction  

/////

function av=centroidAverageDiffer(centroidsVect)
    sumDif=0;
    len = length(centroidsVect);
    for i=2:len
      sumDif=sumDif + abs(centroidsVect(i) - centroidsVect(i-1));
    end
    av=sumDif/(len-1);
endfunction 

function f=flux(centroidsVect)
    f=[];
    len = length(centroidsVect);
    for i=2:len
      f(i-1) = abs(centroidsVect(i) - centroidsVect(i-1));
    end
endfunction 


//##########Wave##############

//Main
function analizeFileByWave(wave, times, step, start)
  
  //start = round(length(wave)/2);
  //sumDif=0;  sumA=0;  sumZ=0;  sumEnergy = 0;
  
  wavAmpl=[];
  wavDif=[];  
  zero=[];
  energy=[];  
  
  for i=1:times
    dif=0;
    w = wave((start +(i-1)*times):(start+(step*i-1)));
    for j=2:length(w)
      dif=dif + abs(w(j)- w(j-1));
    end,
    ampl = amplitude(w);
    ampl2 = ifZeroThenOne(ampl);
    wavDif(i) = (dif/step)/ampl2;
    //sumDif = sumDif + wavDif(i);
    wavAmpl(i) = ampl;
    //sumA = sumA + wavAmpl(i);
    zero(i) = zeroCrossings(w)
    //sumZ = sumZ + zero(i);
    energy(i) = getEnergy(w);
    //sumEnergy = sumEnergy + energy(i);
  end;

  avEnergy = mean(energy);
  less = 0;
  for i=1:times
    if energy(i)<avEnergy then less = less + 1;
    end;
  end;
  lowEnergy = less*100/times;
  
  //d = sumDif/times; - average difference btw 2 points
  //mfprintf(fid, ", %f", sumDif/times);
  mfprintf(fid, ", %f", mean(wavDif));
  mfprintf(fid, ", %f", stdev(wavDif));
  //a = sumA/times; - average amplitude
  //mfprintf(fid, ", %f", sumA/times);
  mfprintf(fid, ", %f", mean(wavAmpl));
  mfprintf(fid, ", %f", stdev(wavAmpl));
  //z = sumZ/times; - average zeroCrossings
  //mfprintf(fid, ", %f", sumZ/times);
  mfprintf(fid, ", %f", mean(zero));
  mfprintf(fid, ", %f", stdev(zero));
  //persentage of analysis windows where energy is less then averageEnergy
  mfprintf(fid, ", %f", lowEnergy);
endfunction
//Main end


function a=amplitude(wave)
  a=max(wave)-min(wave);
endfunction

function a=ifZeroThenOne(n)
  a=n;
  if n==0 then a=1;
  end;
endfunction

function e=getEnergy(vector)
  len = length(vector);
  e=0;
  for i=1:len
    e=e+vector(i)^2;
  end;  
endfunction

//#######FFT#######


//Main
function analizeFileByFft(wave, times, step, start)
  
  //start = round(length(wave)/2);
  //sumLowF = 0;  sumHighF = 0;  sumMaxH = 0;  sumMaxL = 0;  sum85 = 0;  rolloff = 0;
  
  low = [];
  high = [];
  maxH = [];
  maxL = [];
  summ85 = [];
  hl = [];
  maxHL = [];
  
  for i=1:times
    w = wave((start +(i-1)*times):(start+(step*i-1)));
    fft = waveFft(w);
    low(i) = waveFftL(fft);
    //sumLowF = sumLowF + low(i);
    high(i) = waveFftH(fft);
    //sumHighF = sumHighF + high(i);
    maxH(i) = waveMaxH(fft);
    //sumMaxH = sumMaxH + maxH(i);
    maxL(i) = waveMaxL(fft);
    //sumMaxL = sumMaxL + maxL(i);
    sum85 = (sum(low)+sum(high))*0.85;
    summ85(i) = sum85;
    //rolloff = rolloff + getRolloff(fft, sum85);
    
    maxHL(i) = maxH(i)+maxL(i);
    hl(i) = low(i)+high(i);
    
  end
  

  //avLowF = sumLowF/times;
  //mfprintf(fid, ", %f", sumLowF/times);
  mfprintf(fid, ", %f", mean(low));
  mfprintf(fid, ", %f", stdev(low));
  //avHighF = sumHighF/times;
  //mfprintf(fid, ", %f", sumHighF/times);
  mfprintf(fid, ", %f", mean(high));
  mfprintf(fid, ", %f", stdev(high));
  //avHighLow = (sumHigh/sumLow)/times;
  //mfprintf(fid, ", %f", (sumHighF/sumLowF)/times);
  mfprintf(fid, ", %f", mean(hl));
  mfprintf(fid, ", %f", stdev(hl));
  
  //maxH = sumMaxH/times;
  //mfprintf(fid, ", %f", sumMaxH/times);
  mfprintf(fid, ", %f", mean(maxH));
  mfprintf(fid, ", %f", stdev(maxH));
  //maxL = sumMaxL/times;
  //mfprintf(fid, ", %f", sumMaxL/times);
  mfprintf(fid, ", %f", mean(maxL));
  mfprintf(fid, ", %f", stdev(maxL));
  //maxHL = (sumMaxH/sumMaxL)/times;
  //mfprintf(fid, ", %f", (sumMaxH/sumMaxL)/times);
  mfprintf(fid, ", %f", mean(maxHL));
  mfprintf(fid, ", %f", stdev(maxHL));
  //rolloff
  //mfprintf(fid, ", %f", rolloff/times);
  mfprintf(fid, ", %f", mean(summ85));
  mfprintf(fid, ", %f", stdev(summ85));
  
endfunction
//Main end

function f=getRolloff(fftvector, sum85)
   sumR = 0;
   f=0;
   for i=1:256
      sumR = sumR + fftvector(i);
      if(sumR>sum85) then 
        f=i;
        break;
      end;
   end
endfunction

function f=waveFft(wave)
    f = abs(fft(wave));
    f = f(1:256);
endfunction

//normalized by fft amplitude
function l=waveFftL(fftWave)
    s = round(length(fftWave)/2);
    l=sum(fftWave(1:s))/ifZeroThenOne(amplitude(fftWave));
endfunction

//normalized by fft amplitude
function l=waveFftH(fftWave)
    s = round(length(fftWave)/2);
    l = sum(fftWave((s+1):(s*2)))/ifZeroThenOne(amplitude(fftWave));
endfunction

function f=waveFftHL(avH, avL)
    f=avH/avL;
endfunction

function m=waveMaxH(fftWave)
    s = round(length(fftWave)/2);
    m = max(fftWave((s+1):(s*2)));
endfunction

function m=waveMaxL(fftWave)
    s = round(length(fftWave)/2);
    m=max(fftWave(1:s));
endfunction

function f=waveMaxHL(maxH, maxL)
    f=maxH/maxL;
endfunction

//#################################

function p=zeroCrossings(vector)
    s = length(vector);
    p=0;
    for i=1:s-1
      if vector(i)==0 then
        p=p+1;
      elseif vector(i)>0 then
        if vector(i+1)<0 then 
          p=p+1;
        end,
      elseif vector(i)<0 then
        if vector(i+1)>0 then
          p=p+1;
        end,
      end,
    end
endfunction

//###### Rythm ############################

//Main
function analizeFileByRythm(wave, i, rstep)
  w = wave(50000:$);
  w = w((i-1)*rstep+1:i*rstep);
  w = findPeak(getACor(w));
  len = length(w);
  if len==0 then b=0; v=0;
  else
    sum = sum(w(2,1:$));
    w = convert1Dim_sort(w)
    p0 = getPeriod(w,0);
    a0 = getAmplitude(w, sum, 0);
    p1 = getPeriod(w,1);
    a1 = getAmplitude(w, sum, 1);
    p2 = getPeriod(w,2);
    a2 = getAmplitude(w, sum, 2);
    p3 = getPeriod(w,3);
    a3 = getAmplitude(w, sum, 3);
  end;

  mfprintf(fid, ", %f", a0);
  mfprintf(fid, ", %f", p0);  
  mfprintf(fid, ", %f", a1);
  mfprintf(fid, ", %f", p1); 
  mfprintf(fid, ", %f", a2);
  mfprintf(fid, ", %f", p2); 
  mfprintf(fid, ", %f", a3);
  mfprintf(fid, ", %f", p3); 

endfunction

function w=getACor(y)
  //y = wavread(fileName);
  ys = y(1,1:8:$);
  w = corr(ys, 2000);
endfunction

function result=findPeak(w)
  len = length(w);
  result = [2,];
  k=1;
  c=0;
  //pervyje 25 ne v s4et
  w = w(26:$);
  //wm = w(51:length(w)-50);
  for i=1:round((len-125)/5)
    wm = w(51:length(w)-50);
    [mx, n] = max(wm);
    c = control(w,n+50);
    if c==1 then 
        result(1,k)=n+50;
        result(2,k)=w(n+50);
        k=k+1;
        if k>5 then break; end;
    end;
    w=zeroW(w,n);
    //wm=zeroWm(wm,n);
  end;
endfunction

function p=control(w,n)
  len = length(w);
  point = w(n);
  for i=(n-50):(n+50)
    if i==n then //
    else
      if (w(i)>point) then 
        p=0;
        break; 
      else
        p=1;
      end;
    end;
  end;
endfunction;

function b=getPeriod(a, i)
  if i==0 then b = round(a(1));
  else b=round(a(i+1))/round(a(i));
  end;
endfunction

function v=getAmplitude(a, sum, i)
  if sum==0 then v=0;
  else
    if i==0 then v = (a(1)-round(a(1)))/sum;
    else v=(a(i+1)-round(a(i+1)))/sum;
    end;
  end;  
endfunction;

function k=convert1Dim_sort(w)
  len = length(w)/2;
  k=[];
  for j=1:len
    k(j)=w(1,j)+w(2,j); 
  end;    
  k=mtlb_sort(k);
endfunction;

function w=zeroW(w, n)
  for j=n-5:n+5 w(j+50)=0; end;    
endfunction;

  
  
